package RevenueManagement.CrashCourse;

import java.io.*;
import java.util.*;

public class RobotReader
{
    // a buffered reader is used to read a file line-by-line
    BufferedReader bur;
    // a string tokenizer is used to split a line into tokens
    StringTokenizer tokenizer;
    // We will refer to these characters as deliminators and will not read them
    String dels;
    
    public RobotReader( File inFile )
    {
        try
        {
            bur = new BufferedReader ( new InputStreamReader ( new FileInputStream ( inFile ) ) );
            // we treat spaces, tabs and square brackets as deliminators
            dels = " \t []";            
        }
        catch ( Exception e )
        {
            throw new Error ( "exception thrown " + e.getClass() + " " + e.getMessage() );
        }
    }
    
    
    // reads the robots from the input file, creates them and adds them to the robot keeper
    // assumes that a robot keeper has already been created
    public void read()
    {
        try
        {
            String line = bur.readLine();
            while ( line != null )
            {
                // tokenizer is getting ready to tokenize the line that has just been read
                tokenizer = new StringTokenizer( line , dels );
                // extract the robot name
                String name = tokenizer.nextToken();
                // extract the health level
                // note that string tokenizer always returns a string
                // we need to convert the string into integer
                String healthString = tokenizer.nextToken();
                int health = Integer.parseInt( healthString );
                // create the robot and add it to robot keeper
                Robot robotCreated = new Robot( name , health );
                RobotKeeper.getInstance().addRobot( robotCreated );
                // read next line
                line = bur.readLine();
            }
        }
        catch ( IOException e )
        {
            throw new Error ( "exception thrown " + e.getClass() + " " + e.getMessage() );
        }
    }
}
