package RevenueManagement.MultiLeg.BidPriceLP;

import RevenueManagement.MultiLeg.General.*;

public class CapacityKeeper
{
    /* indices of the array are the origin and destinations of the flights */
    int[][] capacities;
    static CapacityKeeper instance;
    
    public CapacityKeeper()
    {
        capacities = new int[ LocationKeeper.instance().getNoLocations() ][ LocationKeeper.instance().getNoLocations() ];
        instance = this;
    }
    
    /* set capacities so that they reflect initial flight capacities */
    public void initialize()
    {
        for ( int o = 0 ; o < LocationKeeper.instance().getNoLocations() ; o ++ )
        {
            for ( int d = 0 ; d < LocationKeeper.instance().getNoLocations() ; d ++ )
            {
                String origin = LocationKeeper.instance().getLocation( o );
                String destination = LocationKeeper.instance().getLocation( d );
                Flight flight = FlightKeeper.instance().getFlight( origin , destination );
                if ( flight != null )
                {
                    capacities[ o ][ d ] = flight.getCapacity();
                }
            }
        }
    }
    
    public int getCapacity ( Flight flight )
    {
        int origin = LocationKeeper.instance().getLocationIndex( flight.getOrigin() );
        int destination = LocationKeeper.instance().getLocationIndex( flight.getDestination() );
        return capacities[ origin ][ destination ];
    }
    
    /* after selling the itinierary itin, adjust the flight leg capacities */
    /* this is useful in a simulation environment */
    public void adjust ( Itin itin )
    {
        Flight[] flights = itin.getFlights();
        for ( int f = 0 ; f < flights.length ; f ++ )
        {
            Flight flight = flights[ f ];
            int origin = LocationKeeper.instance().getLocationIndex( flight.getOrigin() );
            int destination = LocationKeeper.instance().getLocationIndex( flight.getDestination() );
            capacities[ origin ][ destination ] --;
        }
    }
    
    public static CapacityKeeper instance()
    {
        return instance;
    }
    
}
