package RevenueManagement.MultiLeg.BidPriceLP;

import RevenueManagement.MultiLeg.General.*;

public class Simulator
{
    int noIterations;
    /* we use this array to grab the dual variables from bid price lp */
    double[] duals;
    static Simulator instance;
    
    public Simulator ( int noIterations_ )
    {
        noIterations = noIterations_;
        duals = new double[ FlightKeeper.instance().getNoFlights() ];
        instance = this;
    }
    
    public void simulate()
    {
        double totalRevenue = 0;
        for ( int it = 0 ; it < noIterations ; it ++ )
        {
            double revenue = 0;
            /* initialize capacity keeper so that it reflects initial capacities of flights */
            CapacityKeeper.instance().initialize();
            System.out.print( it + " " );
            for ( int time = 0 ; time < GlobalParamsKeeper.instance().getNoTimePeriods() ; time ++ )
            {
                /* sample an itinerary request */
                Itin sample = ProbabilityKeeper.instance().sample( time );
                /* check if we have available capacity to serve the itinerary */
                boolean available = checkCapacities( sample );
                /* if we have capacity, then solve the bid price LP to get bid prices */
                /* if we do not have the capacity, we reject the itinerary request anyway */
                if ( available )
                {
                    try
                    {
                        BidPriceLP.instance().construct( time );
                        BidPriceLP.instance().solve();
                        if ( time == 0 )
                        {
                            System.out.println( "Upper Bound on Expected Revenue " + BidPriceLP.instance().getLP().get_objval() );
                        }
                        BidPriceLP.instance().getLP().get_pi_array( duals );
                        double bidPrices = addBidPrices( duals , sample );
                        /* if the sum of the fare exceeds the sum of the bid prices, then sell */
                        if ( sample.getFare() >= bidPrices )
                        {
                            /* itinerary is sold so update objective and revenue */
                            revenue += sample.getFare();
                            CapacityKeeper.instance().adjust( sample );
                        }
                    }
                    catch ( Exception e )
                    {
                        throw new Error ( e.getClass() + " " + e.getMessage() );
                    }
                }
            }
            GlobalParamsKeeper.instance().getOutWriter().println( revenue );
            totalRevenue += revenue;
            GlobalParamsKeeper.instance().getOutWriter().flush();
        }
        System.out.println( "Average Revenue " + ( totalRevenue / noIterations ) );
    }
    
    /* checks if there is enough capacity to cover an itinerary */
    public boolean checkCapacities ( Itin itin )
    {
        Flight[] flights = itin.getFlights();
        for ( int f = 0 ; f < flights.length ; f ++ )
        {
            if ( CapacityKeeper.instance().getCapacity( flights[ f ] ) <= 0 )
                return false;
        }
        return true;
    }
    
    /* adds up the bid prices associated with the flights in an itinerary */
    /* duals, i.e. bid prices, are kept in the array */
    public double addBidPrices( double[] duals , Itin itin )
    {
        double sum = 0;
        Flight[] flights = itin.getFlights();
        for ( int f = 0 ; f < flights.length ; f ++ )
        {
            /* get constraint index corresponding to this flight */
            int flindex = FlightKeeper.instance().getFlightIndex( flights[ f ] );
            sum -= duals[ flindex ];
        }
        return sum;
    }
    
    public static Simulator instance()
    {
        return instance;
    }
    
}
