package RevenueManagement.MultiLeg.General;

/* assumes that there is at most one flight between an origin destination pair */
/* each flight is indexed by an integer, this is useful for the linear programming formulation */
public class FlightKeeper
{
    /* array's indices are origin and destination index of the flight */
    Flight[][] flights;
    int[][] flightIndices;
    int size;
    static FlightKeeper instance;
    
    public FlightKeeper( int noLocations )
    {
        flights = new Flight[ noLocations ][ noLocations ];
        flightIndices = new int[ noLocations ][ noLocations ];
        instance = this;
    }
    
    public void addFlight ( Flight flight )
    {
        /* get origin and destination indices for the flight */
        int orIndex = LocationKeeper.instance().getLocationIndex( flight.getOrigin() );
        int desIndex = LocationKeeper.instance().getLocationIndex( flight.getDestination() );
        /* add the flight in the appropriate place */
        flights[ orIndex ][ desIndex ] = flight;
        flightIndices[ orIndex ][ desIndex ] = size;
        size ++;
    }
    
    /* returns the flight between a particular origin destination */
    /* returns null is there is no such flight */
    public Flight getFlight ( String origin , String destination )
    {
        int orIndex = LocationKeeper.instance().getLocationIndex( origin );
        int desIndex = LocationKeeper.instance().getLocationIndex( destination );
        return flights[ orIndex ][ desIndex ];
    }
    
    /* returns the flight with a particular index */
    /* extremely inefficient, do not overuse */
    public Flight getFlight ( int index )
    {
        for  ( int i = 0 ; i < flights.length ; i ++ )
        {
            for ( int j = 0 ; j < flights[ i ].length ; j ++ )
            {
                if ( ( flights[ i ][ j ] != null ) && ( index == flightIndices[ i ][ j ] ) )
                {
                    return flights[ i ][ j ];
                }
            }
        }
        throw new Error ( "cannot find the itin with index " + index );
    }
    
    /* returns the index of the flight between a particular origin destination */
    /* returns -1 is there is no such flight */
    public int getFlightIndex ( String origin , String destination )
    {
        int orIndex = LocationKeeper.instance().getLocationIndex( origin );
        int desIndex = LocationKeeper.instance().getLocationIndex( destination );
        if ( getFlight ( origin , destination ) != null )
        {
            return flightIndices[ orIndex ][ desIndex ];
        }
        else
        {
            return -1;
        }         
    }
    
    /* returns the index of a flight  */
    /* returns -1 is there is no such flight */
    public int getFlightIndex ( Flight flight )
    {
        String origin = flight.getOrigin();
        String destination = flight.getDestination();
        return getFlightIndex( origin , destination );
    }
    
    public int getNoFlights()
    {
        return size;
    }
    
    public static FlightKeeper instance()
    {
        return instance;
    }
}
