package RevenueManagement.MultiLeg.General;

/* assumes that there is at most one itin between each origin destination and for each fare class */
/* each itin is indexed by an integer, this is useful for the linear programming formulation */
public class ItinKeeper
{
    /* array's indices are origin destination and fare class */
    Itin[][][] itins;
    int[][][] itinIndices;
    int size;
    static ItinKeeper instance;
    
    public ItinKeeper( int noLocations , int noFareClasses )
    {
        itins = new Itin[ noLocations ][ noLocations ][ noFareClasses ];
        itinIndices = new int[ noLocations ][ noLocations ][ noFareClasses ];
        instance = this;
    }
    
    public void addItin ( Itin itin )
    {
        /* get origin and destination indices for the flight */
        int orIndex = LocationKeeper.instance().getLocationIndex( itin.getOrigin() );
        int desIndex = LocationKeeper.instance().getLocationIndex( itin.getDestination() );
        int fareClass = itin.getFareClass();
        /* add the itin in the appropriate place */
        itins[ orIndex ][ desIndex ][ fareClass ] = itin;
        itinIndices[ orIndex ][ desIndex ][ fareClass ]= size;
        size ++;
    }
    
    /* returns the itin between a particular origin destination and fare class */
    /* returns null is there is no such itin */
    public Itin getItin ( String origin , String destination , int fareClass )
    {
        int orIndex = LocationKeeper.instance().getLocationIndex( origin );
        int desIndex = LocationKeeper.instance().getLocationIndex( destination );
        return itins[ orIndex ][ desIndex ][ fareClass ];
    }
    
    /* returns the itin with a particular index */
    /* this is extremely inefficient, do not overuse */
    public Itin getItin ( int index )
    {
        for  ( int i = 0 ; i < itins.length ; i ++ )
        {
            for ( int j = 0 ; j < itins[ i ].length ; j ++ )
            {
                for ( int k = 0 ; k < itins[ i ][ j ].length ; k ++ )
                {
                    if ( ( itins[ i ][ j ][ k ] != null ) && ( index == itinIndices[ i ][ j ][ k ] ) )
                    {
                        return itins[ i ][ j ][ k ];
                    }
                }
            }
        }
        throw new Error ( "cannot find the itin with index " + index );
    }
    
    /* returns the index of the itin between a particular origin destination and fare class*/
    /* returns -1 is there is no such itin */
    public int getItinIndex ( String origin , String destination , int fareClass )
    {
        int orIndex = LocationKeeper.instance().getLocationIndex( origin );
        int desIndex = LocationKeeper.instance().getLocationIndex( destination );
        if ( getItin ( origin , destination , fareClass ) != null )
        {
            return itinIndices[ orIndex ][ desIndex ][ fareClass ];
        }
        else
        {
            return -1;
        }         
    }
    
    /* returns the index of an itinerary */
    public int getItinIndex( Itin itin )
    {
        return getItinIndex ( itin.getOrigin() , itin.getDestination() , itin.getFareClass() );
    }
    
    public int getNoItins()
    {
        return size;
    }
    
    public static ItinKeeper instance()
    {
        return instance;
    }
}
