package RevenueManagement.MultiLeg.General;

/* keeps the probability that we are going to see a request for a particular itinerary at a particular time period */
public class ProbabilityKeeper
{
    /* array's indices are no of time periods and itineraries */
    double[][] probabilities;
    /* useful for generating samples of the itinerary requests */
    double[][] cdf;
    static ProbabilityKeeper instance;
    
    public ProbabilityKeeper( int noTimePeriods , int noItins )
    {
        probabilities = new double[ noTimePeriods ][ noItins ];
        instance = this;
    }
    
    public void addProbability ( int time , Itin itin , double p )
    {
        int itinIndex = ItinKeeper.instance().getItinIndex( itin );
        probabilities[ time ][ itinIndex ] = p;
    }
    
    public double getProbability ( int time , Itin itin )
    {
        int itinIndex = ItinKeeper.instance().getItinIndex( itin );
        return probabilities[ time ][ itinIndex ];
    }
    
    /* constructs the cdf of itinerary arrivals from the pdf */
    /* useful for sampling */
    public void contructCDF()
    {
        cdf = new double[ GlobalParamsKeeper.instance().getNoTimePeriods() ][ ItinKeeper.instance().getNoItins() ];
        for ( int t = 0 ; t < GlobalParamsKeeper.instance().getNoTimePeriods() ; t ++ )
        {
            cdf[ t ][ 0 ] = probabilities[ t ][ 0 ];
            for ( int i = 1 ; i < ItinKeeper.instance().getNoItins() ; i ++ )
            {
                cdf[ t ][ i ] = cdf[ t ][ i - 1 ] + probabilities[ t ][ i ];
            }
            if ( Math.abs( cdf[ t ][ ItinKeeper.instance().getNoItins() - 1 ] - 1 ) > 1e-4 )
            {
                throw new Error ( "pdf does not add up to 1" );
            }
        }
    }
    
    /* generates a sample of the itinerary arrival at time t */
    public Itin sample ( int time )
    {
        double r = Math.random();
        for ( int i = 0 ; i < cdf[ time ].length ; i ++ )
        {
            if ( r < cdf[ time ][ i ] )
            {
                 return ItinKeeper.instance().getItin( i );       
            }
        }
        throw new Error ( "this line should not be executed" );
    }
    
    public static ProbabilityKeeper instance()
    {
        return instance;
    }
}