package RevenueManagement.MultiLeg.General;

import java.util.*;
import java.io.*;


public class ProblemGenerator
{
    int noTimePeriods;
    int noLocations;
    /* fare for first class is distance bewteen o-d, fare for second class is fareDifference times more */
    /* we always generate two fare classes */
    int fareDifference;
    /* nominal value is 1, in which case average demand for a flight is equal to capacity */
    double congestion;
    File outFile;
    /* keeps coordinates of locations */
    int[][] locations;
    /* keeps a probability value for each o-d pair */
    /* this probability is allocated between different fares at different points in time */
    double[][] baseProbabilities;
    /* what time period hight revenue fare class starts kicking in */
    /* this changes some time between 1/2 was and 2/3 way into horizon */
    double[][] timeTreshold;
    /* loads on flights that enter and leave hub */
    double[] flightLoadIn;
    double[] flightLoadOut;
    Random random;
    PrintWriter pw;
    
    public ProblemGenerator ( int noTimePeriods_ , int noLocations_ , int fareDifference_ , double congestion_ , File outFile_ )
    {
        noTimePeriods = noTimePeriods_;
        noLocations = noLocations_;
        fareDifference = fareDifference_;
        congestion = congestion_;
        outFile = outFile_;
        
        random = new Random( 1L );
        
        try
        {
            pw = new PrintWriter ( new BufferedWriter ( new OutputStreamWriter ( new FileOutputStream ( outFile ) ) ) );
        }
        catch ( IOException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() );
        }
    }
    
    
    public void generate()
    {
        /* location 0 is hub */
        locations = new int[ noLocations + 1 ][ 2 ];
        locations[ 0 ][ 0 ] = 50;
        locations[ 0 ][ 1 ] = 50;
        for ( int i = 1 ; i < noLocations + 1 ; i ++ )
        {
            locations[ i ][ 0 ] = (int) ( random.nextDouble() * 100 );
            locations[ i ][ 1 ] = (int) ( random.nextDouble() * 100 );
        }
        
        double sum = 0;
        baseProbabilities = new double[ noLocations + 1 ][ noLocations + 1 ];
        timeTreshold = new double[ noLocations + 1 ][ noLocations + 1 ];
        for ( int o = 0 ; o < noLocations + 1 ; o ++ )
        {
            for ( int d = 0 ; d < noLocations + 1 ; d ++ )
            {
                if ( o != d )
                {
                    baseProbabilities[ o ][ d ] = random.nextDouble();
                    timeTreshold[ o ][ d ] = ( ( 0.167 * random.nextDouble() ) + 0.5 ) * noTimePeriods;
                    sum += baseProbabilities[ o ][ d ];
                }
            }
        }
        
        for ( int o = 0 ; o < noLocations + 1 ; o ++ )
        {
            for ( int d = 0 ; d < noLocations + 1 ; d ++ )
            {
                if ( o != d )
                    baseProbabilities[ o ][ d ] = baseProbabilities[ o ][ d ] / sum;
            }
        }
        
        flightLoadOut = new double[ noLocations + 1 ];
        flightLoadIn = new double[ noLocations + 1 ];
        
        for ( int o = 0 ; o < noLocations + 1 ; o ++ )
        {
            for ( int d = 0 ; d < noLocations + 1 ; d ++ )
            {
                if ( o != d )
                {
                    double load = baseProbabilities[ o ][ d ] * noTimePeriods;
                    if ( o != 0 )
                        flightLoadIn[ o ] += baseProbabilities[ o ][ d ] * noTimePeriods;
                    if ( d != 0 )
                        flightLoadOut[ d ] += baseProbabilities[ o ][ d ] * noTimePeriods;
                }
            }
        }
        
        pw.println( "# number of time periods" );
        pw.println( noTimePeriods );
        pw.println();
        
        pw.println( "# locations - simple list" );
        pw.println( "# first line is number of locations" );
        pw.println( noLocations + 1 );
        for ( int i = 0 ; i < noLocations + 1 ; i ++ )
        {
            pw.println( ( 100 + i ) );
        }
        pw.println();
        
        pw.println( "# flights - from to capacity" );
        pw.println( "# first line is number of flights" );
        pw.println( noLocations * 2 );
        
        for ( int o = 1 ; o < noLocations + 1 ; o ++ )
            pw.println( ( 100 + o ) + " 100 " + (int) Math.round( flightLoadIn[ o ] / congestion ) );
        
        for ( int d = 1 ; d < noLocations + 1 ; d ++ )
            pw.println( "100 " + ( 100 + d ) + " " + (int) Math.round( flightLoadOut[ d ] / congestion ) );
        
        pw.println();
        
        pw.println( "# itineraries - from, to, class, fare, no flights in itinerary, list of flights in itinerary" );
        pw.println( "# first line is number of itineraries and no fare classes" );       
        pw.println( 2 * noLocations * ( noLocations + 1 ) + " " + 2 );
        
        for ( int o = 0 ; o < noLocations + 1 ; o ++ )
        {
            for ( int d = 0 ; d < noLocations + 1 ; d ++ )
            {
                if ( o != d )
                {
                    double dist = distance( o , d );
                    pw.print( ( 100 + o ) + " " + ( 100 + d ) + " 0 " + dist + " " );
                    if ( ( o != 0 ) && ( d != 0 ) )
                    {
                        pw.println( 2 + " [ " + ( 100 + o ) + " 100 ] [ 100 " + ( 100 + d ) + " ]" );
                    }
                    else
                    {
                        pw.println( 1 + " [ " + ( 100 + o ) + " " + ( 100 + d ) + " ]" );
                    }
                    
                    pw.print( ( 100 + o ) + " " + ( 100 + d ) + " 1 " + dist * fareDifference + " " );
                    if ( ( o != 0 ) && ( d != 0 ) )
                    {
                        pw.println( 2 + " [ " + ( 100 + o ) + " 100 ] [ 100 " + ( 100 + d ) + " ]" );
                    }
                    else
                    {
                        pw.println( 1 + " [ " + ( 100 + o ) + " " + ( 100 + d ) + " ]" );
                    }
                }
            }
        }
        pw.println();
        
        pw.println( "# probabilities - time period itinerary probability" );
        
        for ( int t = 0 ; t < noTimePeriods ; t ++ )
        {
            pw.print( t + "\t" );
            for ( int o = 0 ; o < noLocations + 1 ; o ++ )
            {
                for ( int d = 0 ; d < noLocations + 1 ; d ++ )
                {
                    if ( o != d )
                    {
                        double p = baseProbabilities[ o ][ d ]; 
                        double p1 = findLowProbability( t , o , d );
                        double p2 = findHighProbability( t , o , d );
                        pw.print( "[ " + ( 100 + o ) + " " + ( 100 + d ) + " 0 ]\t" + ( p * p1 / ( p1 + p2 ) ) + "\t" );
                        pw.print( "[ " + ( 100 + o ) + " " + ( 100 + d ) + " 1 ]\t" + ( p * p2 / ( p1 + p2 ) ) + "\t" );
                    }
                }
            }
            pw.println();
        }
        pw.flush();
    }
    
    
    /* find arrival probability for low paying fare */
    public double findLowProbability ( int t , int o , int d )
    {
        return baseProbabilities[ o ][ d ] * ( noTimePeriods - t ) / noTimePeriods;
    }
    
    
    /* find arrival probability for high paying fare */
    public double findHighProbability ( int t , int o , int d )
    {
        double tresh = timeTreshold[ o ][ d ];
        if ( t < tresh )
            return 0;
        return baseProbabilities[ o ][ d ] * ( t - tresh ) / ( noTimePeriods - tresh );
    }
    
    
    public int distance( int o , int d )
    {
        return (int) Math.pow( Math.pow( locations[ o ][ 0 ] - locations[ d ][ 0 ] , 2 ) + Math.pow( locations[ o ][ 1 ] - locations[ d ][ 1 ] , 2 ) , 0.5 );
    }
    
    
    public static void main ( String args[] )
    {
        File outFile = new File ( "c:\\huseyin\\research\\RevenueManagement\\MultiLeg\\Data\\rm_100_2_1.6_8.0.txt" );
        int noTimePeriods = 100;
        int noLocations = 2;
        double congestion = 1.6;
        int fareDifference = 8;
        
        ProblemGenerator pg = new ProblemGenerator( noTimePeriods ,  noLocations , fareDifference , congestion , outFile );
        pg.generate();   
    } 
}