package RevenueManagement.MultiLeg.General;

import java.util.*;
import java.io.*;

public class ProblemReader
{
    File inFile;
    BufferedReader bur;
    String dels = "\t[] ";
    String comment = "#";
    
    public ProblemReader ( File inFile_ )
    {
        inFile = inFile_;
        try
        {
            bur = new BufferedReader( new InputStreamReader( new FileInputStream( inFile ) ) );
        }
        catch ( IOException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() );
        }
    }
    
    
    public void read()
    {
        try
        {
            /* read output file name */
            String line = skipComments();
            String outFile = line;
            /* read number of time periods */
            line = skipComments();
            StringTokenizer st = new StringTokenizer( line , dels );
            int noTimePeriods = Integer.parseInt( st.nextToken() );
            
            new GlobalParamsKeeper( new File( outFile ) , noTimePeriods );
            
            /* read locations */
            line = skipComments();
            int noLocations = Integer.parseInt( line );
            new LocationKeeper( noLocations );
            
            for ( int i = 0 ; i < noLocations ; i ++ )
            {
                line = skipComments();
                st = new StringTokenizer( line , dels );
                LocationKeeper.instance().addLocation( st.nextToken() );
            }
            
            /* read flights */
            line = skipComments();
            int noFlights = Integer.parseInt( line );
            new FlightKeeper( noFlights );
            
            for ( int f = 0 ; f < noFlights ; f ++ )
            {
                line = skipComments();
                st = new StringTokenizer( line , dels );
                String origin = st.nextToken();
                String dest = st.nextToken();
                int capacity = Integer.parseInt( st.nextToken() );
                Flight fli = new Flight( origin , dest , capacity );
                FlightKeeper.instance().addFlight( fli );
            }
            
            /* read itineraries */
            line = skipComments();
            st = new StringTokenizer( line , dels );
            int noItins = Integer.parseInt( st.nextToken() );
            int noFareClasses = Integer.parseInt( st.nextToken() );
            new ItinKeeper( noLocations , noFareClasses );
            
            for ( int i = 0 ; i < noItins ; i ++ )
            {
                line = skipComments();
                st = new StringTokenizer( line , dels );
                String origin = st.nextToken();
                String dest = st.nextToken();
                int fareClass = Integer.parseInt( st.nextToken() );
                double fare = Double.parseDouble( st.nextToken() );
                int noflis = Integer.parseInt( st.nextToken() );
                Itin itin = new Itin( noflis , origin , dest , fareClass , fare );
                ItinKeeper.instance().addItin( itin );
                /* read the flights included in the itinerary */
                for ( int f = 0 ; f < noflis ; f ++ )
                {
                    origin = st.nextToken();
                    dest = st.nextToken();
                    Flight flight = FlightKeeper.instance().getFlight( origin , dest );
                    itin.addFlight( flight );
                }
            }
            
            /* read itinerary arrival probabilities */
            new ProbabilityKeeper( noTimePeriods , noItins );
            for ( int t = 0 ; t < noTimePeriods ; t ++ )
            {
                line = skipComments();
                st = new StringTokenizer( line , dels );
                int time = Integer.parseInt( st.nextToken() );
                for ( int it = 0 ; it < noItins ; it ++ )
                {
                    String origin = st.nextToken();
                    String dest = st.nextToken();
                    int fareClass = Integer.parseInt( st.nextToken() );
                    double prob = Double.parseDouble( st.nextToken() );
                    Itin itin = ItinKeeper.instance().getItin( origin , dest , fareClass );
                    ProbabilityKeeper.instance().addProbability( time , itin , prob );
                }
            }
            ProbabilityKeeper.instance().contructCDF();
        }
        catch ( NumberFormatException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() );
        }
    }
    
    
    protected String skipComments()
    {
        String line = null;
        try
        {
            line = bur.readLine();
            while ( ( line != null ) && ( ( line.startsWith ( comment ) ) || ( line.equals ("" ) ) ) )
                line = bur.readLine();

            return line;
        }
        catch ( IOException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() + " " + line );
        }
    }
}
