package RevenueManagement.QSOptDemo;

import qs.*;
import java.io.*;

public class QSOptDemo
{
    
    public static void main ( String[] args )
    {
        try
        {
            Problem lp = new Problem( "Test Problem" );

            /* we want to maximize the objective */
            lp.change_objsense ( QS.MAX );

            /* build right hand coefficients */
            double[] rhs = new double[ 2 ];
            rhs[ 0 ] = 1.0; rhs[ 1 ] = 3.0;

            /* build sense of the constraints */
            char sense[] = new char[ 2 ];
            sense[ 0 ] = 'L'; sense[ 1 ] = 'L';

            /* the size of this array should be equal to the number of variables */
            /* but it can be all zero's for now */
            int[] varcnt = new int[ 3 ];

            /* add the constraints to the problem */
            /* the first parameter is the number of constraints we are adding */
            lp.add_rows( 2 , varcnt , null , null , null , rhs , sense , null);
            
            /* start adding the objective coefficients of the variables into the linear program */
            /* we have two variables, therefore the first component of the following arrays have dimension two */
            /* this arrays keeps the number of nonzero coefficients of each variable */
            int[][] matcnt_var = new int[ 3 ][];
            int[][] matbeg_var = new int[ 3 ][];
            /* this array keeps constraint index of the nonzero coefficients of each variable */
            int[][] matind_var = new int[ 3 ][];
            /* this array keeps constraint coefficients of the nonzero coefficients of each variable */
            double[][] matval_var = new double[ 3 ][];
            /* the following arrays are the objective function value, upper bound and lower bound, and name of the variables */
            double[][] obj_var = new double[ 3 ][ 1 ];
            double[][] lower_var = new double[ 3 ][ 1 ];
            double[][] upper_var = new double[ 3 ][ 1 ];
            String[][] name_var = new String[ 3 ][ 1 ];
            
            /* add x variable into the problem */
            matcnt_var[ 0 ] = new int[ 1 ];
            matcnt_var[ 0 ][ 0 ] = 2;
            matbeg_var[ 0 ] = new int[ 1 ];
            matbeg_var[ 0 ][ 0 ] = 0;
            /* note x variable has two nonzero coefficients in the constraints */
            matind_var[ 0 ] = new int[ 2 ];
            matind_var[ 0 ][ 0 ] = 0; matind_var[ 0 ][ 1 ] = 1; 
            matval_var[ 0 ] = new double[ 2 ];
            matval_var[ 0 ][ 0 ] = 1.0; matval_var[ 0 ][ 1 ] = 3.0; 
            
            /* add y variable into the problem */
            matcnt_var[ 1 ] = new int[ 1 ];
            matcnt_var[ 1 ][ 0 ] = 2;
            matbeg_var[ 1 ] = new int[ 1 ];
            matbeg_var[ 1 ][ 0 ] = 0;
            /* note y variable has two nonzero coefficients in the constraints */
            matind_var[ 1 ] = new int[ 2 ];
            matind_var[ 1 ][ 0 ] = 0; matind_var[ 1 ][ 1 ] = 1; 
            matval_var[ 1 ] = new double[ 2 ];
            matval_var[ 1 ][ 0 ] = 4.0; matval_var[ 1 ][ 1 ] = -1.0; 
            
            /* add z variable into the problem */
            matcnt_var[ 2 ] = new int[ 1 ];
            matcnt_var[ 2 ][ 0 ] = 1;
            matbeg_var[ 2 ] = new int[ 1 ];
            matbeg_var[ 2 ][ 0 ] = 0;
            /* note z variable has one nonzero coefficients */
            matind_var[ 2 ] = new int[ 1 ];
            matind_var[ 2 ][ 0 ] = 1; 
            matval_var[ 2 ] = new double[ 1 ];
            matval_var[ 2 ][ 0 ] = 1.0; 
                        
            /* construct objective coefficients, lower bounds, upper bounds and names */
            obj_var[ 0 ][ 0 ] = 4.0; obj_var[ 1 ][ 0 ] = 1.0; obj_var[ 2 ][ 0 ] = 3.0;
            lower_var[ 0 ][ 0 ] = 0.0; lower_var[ 1 ][ 0 ] = 0.0; lower_var[ 2 ][ 0 ] = 0.0; 
            upper_var[ 0 ][ 0 ] = 10.0; upper_var[ 1 ][ 0 ] = 20.0; upper_var[ 2 ][ 0 ] = QS.MAXDOUBLE;
            name_var[ 0 ][ 0 ] = "x"; name_var[ 1 ][ 0 ] = "y"; name_var[ 2 ][ 0 ] = "z"; 
            
            /* add variables into problem */
            for ( int i = 0 ; i < 3 ; i ++ )
            {
                lp.add_cols( 1 , matcnt_var[ i ] , matbeg_var[ i ] , matind_var[ i ] , matval_var[ i ] , obj_var[ i ] , lower_var[ i ] , upper_var[ i ] , name_var[ i ] );
            }
            
            /* write the problem to make sure that we did not make a mistake */
            lp.write_lp ( "c:\\test.lp" );
            /* solve lp by using dual simplex */
            lp.opt_dual();
            
            /* get the solution, first create an array with length 3 since we have two variables */
            double[] x = new double[ 3 ];
            lp.get_x_array( x );
            System.out.println( "Current Solution" );
            for ( int ii = 0 ; ii < 3 ; ii ++ )
            {
                System.out.println( x[ ii ] );
            }
            
            /* get the dual variables associated with constraints, first create an array with length 2 since we have two constraints */
            double[] pi = new double[ 2 ];
            lp.get_pi_array( pi );
            System.out.println( "Current Dual Solution" );
            for ( int ii = 0 ; ii < 2 ; ii ++ )
            {
                System.out.println( pi[ ii ] );
            }
        }
        catch ( Exception e )
        {
            throw new Error ( e.getMessage() + " " + e.getClass() );
        }
    }
    
}
