package RevenueManagement.SingleLeg.EMSR;

import RevenueManagement.SingleLeg.General.*;
import java.io.*;

public class EMSRa
{
    
    // keeps zij's that are used to compute the protection levels
    int[][] z;
    // keeps vj's that are used to compute the protection levels
    // vj's are sums of certain zij's
    int[] v;
    static EMSRa instance;
    
    public EMSRa()
    {
        z = new int[ FareClassKeeper.instance().getNoFareClasses() ][  FareClassKeeper.instance().getNoFareClasses() ];
        v = new int[ FareClassKeeper.instance().getNoFareClasses() ];
        instance = this;
    }
    
    
    public void solve()
    {
        // first compute zij's
        for ( int j = FareClassKeeper.instance().getNoFareClasses() - 1 ; j >= 0 ; j -- )
        {
            FareClass fcj = FareClassKeeper.instance().getFareClass( j );
            double pj = fcj.getFare();
            for ( int k = j - 1 ; k >= 0 ; k -- )
            {
                FareClass fck = FareClassKeeper.instance().getFareClass( k );
                double pk = fck.getFare();
                // compute the critical ratio
                double criticalRatio = 1.0 - ( pj / pk );
                z[ j ][ k ] = fck.getCDFInverse( criticalRatio );
            }
        }
        
        // then compute vj's
        for ( int j = FareClassKeeper.instance().getNoFareClasses() - 1 ; j >= 0 ; j -- )
        {
            for ( int k = j - 1 ; k >= 0 ; k -- )
            {
                v[ j ] = v[ j ] + z[ j ][ k ];
            }
        }
    }
    
    
    // given that we are making decisions for fare class j and the capacity is c,
    // how many reservations from fare class j can we accept?
    public int getBookingLimit ( int fareClassNo , int capacity )
    {
        return Math.max( 0 , capacity - v[ fareClassNo ] );
    }
    
    
    public void printProtectionLevels()
    {
        PrintWriter pw = GlobalParamsKeeper.instance().getOutWriter();
        for ( int j = 0 ; j < FareClassKeeper.instance().getNoFareClasses() ; j ++ )
        {
            pw.println( v[ j ] );
        }
        pw.flush();
    }
    
    
    public static EMSRa instance()
    {
        return instance;
    }
}
