package RevenueManagement.SingleLeg.EMSR;

import RevenueManagement.SingleLeg.General.*;
import java.io.*;

public class EMSRb
{
    // keeps the protection levels
    double[] v;
    // keeps the mean demand from different fare classes
    double[] means;
    // keeps the variance of the demand from different fare classes
    double[] variances;
    static EMSRb instance;
    
    public EMSRb()
    {
        v = new double[ FareClassKeeper.instance().getNoFareClasses() ];
        means = new double[ FareClassKeeper.instance().getNoFareClasses() ];
        variances = new double[ FareClassKeeper.instance().getNoFareClasses() ];
        for ( int f = 0 ; f < FareClassKeeper.instance().getNoFareClasses() ;  f ++ )
        {
            FareClass fc = FareClassKeeper.instance().getFareClass( f );
            double[] pdf = fc.getPDF();
            for ( int d = 0 ; d < pdf.length ; d ++ )
            {
                means[ f ] += d * pdf[ d ];
                // store expected value of the square of the random variable first
                variances[ f ] += d * d * pdf[ d ];
            }
            // variance is expected value of the square of the random variable minus
            // the square of the expected value of the random variable
            variances[ f ] = variances[ f ] - means[ f ] * means[ f ];
            System.out.println( variances[ f ] + " " + means[ f ] );
        }
        new NormalTable();
        instance = this;
    }
    
    
    public void solve()
    {
        for ( int f = 1 ; f < FareClassKeeper.instance().getNoFareClasses() ; f ++ )
        {
            FareClass currentFareClass = FareClassKeeper.instance().getFareClass( f );
            double futureDemandMean = 0;
            double futureDemandVariance = 0;
            double futureFare = 0;
            for ( int k = f - 1 ; k >= 0 ; k -- )
            {
                FareClass futureFareClass = FareClassKeeper.instance().getFareClass( k );
                futureDemandMean += means[ k ];
                futureDemandVariance += variances[ k ];
                futureFare += means[ k ] * futureFareClass.getFare();
            }
            futureFare = futureFare / futureDemandMean;
            double criticalRatio = 1.0 - ( currentFareClass.getFare() / futureFare );
            double normInv = NormalTable.instance().getNormalInverse( criticalRatio );
            v[ f ] = futureDemandMean + Math.sqrt( futureDemandVariance ) * normInv;
        }
    }
    
    
    // given that we are making decisions for fare class j and the capacity is c,
    // how many reservations from fare class j can we accept?
    public int getBookingLimit ( int fareClassNo , int capacity )
    {
        return (int) Math.round( Math.max( 0 , capacity - v[ fareClassNo ] ) );
    }
    
    
    public void printProtectionLevels()
    {
        PrintWriter pw = GlobalParamsKeeper.instance().getOutWriter();
        for ( int j = 0 ; j < FareClassKeeper.instance().getNoFareClasses() ; j ++ )
        {
            pw.println( v[ j ] );
        }
        pw.flush();
    }
    
    
    public static EMSRb instance()
    {
        return instance;
    }
}
