package RevenueManagement.SingleLeg.FCFS;

import RevenueManagement.SingleLeg.General.*;
import java.io.*;
import java.util.*;


public class Simulator
{
    // random object is used to generate samples of demands
    Random rand;
    // simulate for this many iterations (or samples)
    int noIterations;
    static Simulator instance;
    
    public Simulator ( int noIterations_ )
    {
        noIterations = noIterations_;
        rand = new Random( 1L );
        instance = this;
    }
    
    
    public void simulate()
    {
        // loop over number of iterations
        for ( int it = 0 ; it < noIterations ; it ++ )
        {
            // initialize capacity and total revenue
            int capacity = GlobalParamsKeeper.instance().getCapacity();
            double revenue = 0;
            // loop over fare classes to see what decisions would have been made
            // last (lowest) fare class arrives first
            for ( int j = FareClassKeeper.instance().getNoFareClasses() - 1 ; j >= 0 ; j -- )
            {
                FareClass fc = FareClassKeeper.instance().getFareClass( j );
                double fare = fc.getFare();
                // generate a random number between 0 and 1
                double randomNumber = rand.nextDouble();
                int demand = fc.getCDFInverse( randomNumber );
                // find how much to sell given the fare class number, remaining  capacity and demand
                int sale = computeSaleQuantity( j , capacity , demand );
                // update capacity and revenue
                capacity = capacity - sale;
                revenue = revenue + fare * sale;
            }
            // iteration is over, dump revenue
            PrintWriter pw = GlobalParamsKeeper.instance().getOutWriter();
            pw.println( revenue );
            pw.flush();
        }
    }
    
    
    // find how much to sell given the fare class number, remaining  capacity and demand
    public int computeSaleQuantity ( int fareClassNo , int capacity , int demand )
    {
        return Math.min( capacity , demand );
    }
    
    
    public static Simulator instance()
    {
        return instance;
    }
}
