package RevenueManagement.SingleLeg.General;

public class FareClass
{
    double fare;
    // the largest possible demand that we can observe from this fare class
    int maxDemand;
    // pdf and cdf's of the demand from this fare class
    double[] pdf;
    double[] cdf;
    
    // construct a fare class with a certain fare
    // we deal with the maximum demand and the probability distribution for the demand later
    public FareClass ( double fare_ )
    {
        fare = fare_;
    }
    
    
    // sets the pdf for this fare class
    // once pdf is set, maximum demand and cdf is automatically computed
    public void setProbability ( double[] pdf_ )
    {
        pdf = pdf_;
        maxDemand = pdf.length - 1;
        // compute cdf using pdf
        cdf = new double[ pdf.length ];
        cdf[ 0 ] = pdf[ 0 ];
        for ( int i = 1 ; i <= maxDemand ; i ++ )
        {
            cdf[ i ] = cdf[ i - 1 ] + pdf[ i ];
        }
        if ( Math.abs( cdf[ maxDemand ] - 1.0 ) > 1e-5 )
        {
            throw new Error ( "pdf values do not add up to 1.0" );
        }
    }
    
    
    public double getFare()
    {
        return fare;
    }
    
    
    public int getMaxDemand()
    {
        return maxDemand;
    }
    
    
    public double[] getPDF()
    {
        return pdf;
    }
    
    
    public double[] getCDF()
    {
        return cdf;
    }
    
    
    // returns the inverse of the cdf at point r
    // useful for EMSR and Littlewood type of computations
    // also useful for generating samples from the demand distribution
    public int getCDFInverse ( double r )
    {
        for ( int i = 0 ; i < cdf.length ; i ++ )
        {
            if ( r < cdf[ i ] )
            {
                return i;
            }
        }
        throw new Error ( "trying to compute the inverse of cdf at point " + r + " which is greater than 1.0" );
    }
}
