package RevenueManagement.SingleLeg.General;

import java.io.*;
import java.util.*;

public class ProblemReader
{
    // file from which we read the problem
    File inFile;
    BufferedReader bur;
    String dels = "\t[] ";
    String comment = "#";
    
    public ProblemReader ( File inFile_ )
    {
        inFile = inFile_;
        try
        {
            bur = new BufferedReader( new InputStreamReader( new FileInputStream( inFile ) ) );
        }
        catch ( IOException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() );
        }
    }
    
    
    // read the problem and automatically populates a fare class keeper, global parameters keeper 
    public void read()
    {
        try
        {
            String line = skipComments();
            StringTokenizer st = new StringTokenizer( line , dels );
            String nt = st.nextToken();
            int noFareClasses = Integer.parseInt( nt );
            new FareClassKeeper( noFareClasses );
            
            line = skipComments();
            st = new StringTokenizer( line , dels );
            nt = st.nextToken();
            int capacity = Integer.parseInt( nt );
            new GlobalParamsKeeper();
            GlobalParamsKeeper.instance().setCapacity( capacity );
            
            // read the fare classes
            for ( int j = 0 ; j < FareClassKeeper.instance().getNoFareClasses() ; j ++ )
            {
                line = skipComments();
                st = new StringTokenizer( line , dels );
                nt = st.nextToken();
                double fare = Double.parseDouble( nt );
                // create the fare class with the read fare and add it to fare class keeeper
                FareClass fc = new FareClass( fare );
                FareClassKeeper.instance().addFareClass( fc );
            }
            
            // read the probabilities for the fare classes
            for ( int j = 0 ; j < FareClassKeeper.instance().getNoFareClasses() ; j ++ )
            {
                line = skipComments();
                st = new StringTokenizer( line , dels );
                nt = st.nextToken();
                // the first token is the maximum demand for the fare class
                // the maximum demand for the fare class can be used to populate the pdf array
                int md = Integer.parseInt( nt );
                double[] pdf = new double[ md + 1 ];
                
                // fill in the pdf array
                for ( int p = 0 ; p < pdf.length ; p ++ )
                {
                    nt = st.nextToken();
                    pdf[ p ] = Double.parseDouble( nt );
                }
                //get the j-th fare class
                FareClass fc = FareClassKeeper.instance().getFareClass( j );
                fc.setProbability( pdf );
            }
            
            // read the output file
            line = skipComments();
            File of = new File ( line );
            GlobalParamsKeeper.instance().setFile( of );
        }
        catch ( NumberFormatException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() );
        }
    }
    
    
    // skips the commented-out and blank lines and returns the first nonempty line
    public String skipComments()
    {
        String line = null;
        try
        {
            line = bur.readLine();
            while ( ( line != null ) && ( ( line.startsWith ( comment ) ) || ( line.equals ("" ) ) ) )
                line = bur.readLine();

            return line;
        }
        catch ( IOException e )
        {
            throw new Error ( e.getClass() + " " + e.getMessage() + " " + line );
        }
    }
}
